/*********************************************************************************************
Sample program for OpenGL vertex command simulation
Promit Roy
Created: 9/27/02
Last Modified: 9/28/02

Driver file for the command emulator. You might feel some deja vu as you see your
very first triangle show up on the screen...again! NOTE: Due to the nature of
triangles, your first triangle may not be quite the same shape as when you originally
met it ;)

Also, if you are new to Direct3D, please please please don't use these vertex commands.
You need to understand the whole process of creating, filling, and rendering vertex
buffers! Don't use this code as a way to avoid learning these things, or you will
_really_ regret it!

Lastly, I don't mind if you use my code unmodified, or if you modify it, or whatever.
If you make some modifications, I'd appreciate being shown the new code, but I'm not
forcing anything. The only thing I want is some credit. It doesn't have to be much; Just
put "Thanks to Promit Roy for the idea" or something similar at the top of the file, 
and maybe a link to the article. Thanks!
*********************************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <d3d8.h>
#include <d3dx8.h>

#include "d3dverts.h"

//Pointer to Direct3D
IDirect3D8* g_pD3D = NULL;
//Pointer to our Direct3D Device
IDirect3DDevice8* g_pDevice = NULL;
//Present parameters
D3DPRESENT_PARAMETERS g_d3dpp;
//Handle to our main(only) window
HWND g_hWnd = NULL;


//Quick input
#define KEYDOWN(Key)		((GetAsyncKeyState( Key ) & 0x8000) == 0x8000)

//There's some float-double conversions which keep throwing warnings. I'm just going to disable them.
#pragma warning(disable:4305)


long CALLBACK WinProc( HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam )
{
	switch( Msg )
	{
	case WM_CREATE:
		return 0;
	case WM_PAINT:
		ValidateRect( g_hWnd, NULL );
		return 0;
	case WM_CLOSE:
		DestroyWindow( g_hWnd );
		PostQuitMessage( 0 );
		return 0;
	default:
		return DefWindowProc( hWnd, Msg, wParam, lParam );
	}
}


int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR lpCmdLine, int nShowCmd )
{
	//Create the window
	WNDCLASSEX wc;

	//name of our window
	const char Name[] = "D3D Vertex Commands";

	wc.cbSize = sizeof( WNDCLASSEX );

	wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.cbClsExtra = wc.cbWndExtra = 0;
	wc.lpfnWndProc = WinProc;
	wc.hInstance = hInstance;
	wc.hbrBackground = (HBRUSH)GetStockObject( GRAY_BRUSH );
	wc.hIcon = wc.hIconSm = LoadIcon( NULL, IDI_APPLICATION );
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.lpszMenuName = NULL;
	wc.lpszClassName = Name;

	RegisterClassEx( &wc );

	g_hWnd = CreateWindowEx( WS_EX_TOPMOST, Name, Name, WS_POPUP | WS_SYSMENU | WS_VISIBLE,
								CW_USEDEFAULT, CW_USEDEFAULT, 640, 480, NULL, NULL,
								hInstance, NULL );
	if( g_hWnd == NULL )
		return -1;

	ShowWindow( g_hWnd, SW_SHOW );
	UpdateWindow( g_hWnd );



	//Now we'll initialize the device
	HRESULT r = 0;

	g_pD3D = Direct3DCreate8( D3D_SDK_VERSION );
	if( g_pD3D == NULL )
		return FALSE;

	g_d3dpp.BackBufferWidth = 640;
	g_d3dpp.BackBufferHeight = 480;
	g_d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;

	g_d3dpp.BackBufferCount = 1;
	g_d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	g_d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;

	g_d3dpp.hDeviceWindow = g_hWnd;
	g_d3dpp.Windowed = FALSE;

	g_d3dpp.EnableAutoDepthStencil = TRUE;
	g_d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	g_d3dpp.FullScreen_RefreshRateInHz = 0;
	g_d3dpp.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

	g_d3dpp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER;

	r = g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_hWnd, 
								D3DCREATE_MIXED_VERTEXPROCESSING, &g_d3dpp, &g_pDevice );
	if( FAILED( r ) )
	{
		return FALSE;
	}

	g_pDevice->Clear( 0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0, 0, 0 ), 1.0, 0 );
	//Build the projection matrix
	D3DXMATRIX matProjection;
	ZeroMemory( &matProjection, sizeof( D3DXMATRIX ) );
	D3DXMatrixPerspectiveFovLH( &matProjection, 90.0, 
				4.0 / 3.0, 0.1, 100.0 );
	g_pDevice->SetTransform( D3DTS_PROJECTION, &matProjection );

	g_pDevice->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_XRGB( 128, 128, 128 ) );
	g_pDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
	g_pDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
	g_pDevice->SetRenderState( D3DRS_SHADEMODE, D3DSHADE_GOURAUD );
	g_pDevice->SetRenderState( D3DRS_LIGHTING, FALSE );


	MSG msg;
	while( true )
	{
		if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			if( msg.message == WM_QUIT )
				break;

			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
		{
			if( KEYDOWN( VK_ESCAPE ) )
				break;

			g_pDevice->Clear( 0, 0, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB( 0, 0, 0 ), 1.0, 0 );
			//Notice how I call g_pDevice->BeginScene() before calling d3dBegin()
			g_pDevice->BeginScene();

			//begin the render sequence
			d3dBegin( D3DPT_TRIANGLELIST, g_pDevice );
				d3dColor4f( 1.0, 1.0, 0.0, 0.0 );
				d3dVertex3f( -0.5, -0.5, 0.5 );

				d3dColor4f( 1.0, 0.0, 1.0, 0.0 );
				d3dVertex3f( 0.0, 0.5, 0.5 );

				d3dColor4f( 1.0, 0.0, 0.0, 1.0 );
				d3dVertex3f( 0.5, -0.5, 0.5 );
			d3dEnd();

			g_pDevice->EndScene();
			g_pDevice->Present( NULL, NULL, NULL, NULL );
		}
	}

	g_pDevice->Release();
	g_pDevice = NULL;

	g_pD3D->Release();
	g_pD3D = NULL;

	return 0;
}